import * as t from '@babel/types';
import type { Metadata } from '../types';
/**
 * Returns `true` if the CSS property or pseudo classes/pseudo elements
 * are defined within a template element (the string pieces of the template literal)
 * rather than in the expression.
 *
 * For example
 * - CSS property: `color: ${({ isPrimary }) => (isPrimary ? 'green' : 'red')};`
 * - Pseudo class: `hover: ${({ isPrimary }) => (isPrimary ? 'color:green' : 'color:red')};`
 * - Pseudo element: `:before {${({ isPrimary }) => (isPrimary ? 'color: green' : 'color:red')}};`
 * @param node
 */
export declare const isCssPropertyInTemplateElement: (node: t.TemplateElement) => boolean;
/**
 * TODO: this is a temporary workaround so that we don't evaluate expressions that may throw an error.
 * It should be removed after addressing https://github.com/atlassian-labs/compiled/issues/1081
 *
 * Returns `true` if
 * - there is another conditional expression at the root of the current template literal
 *   with a conditional expression to evaluate OR
 * - there is a nested logical expression within the current template literal
 *
 * @param node
 * @param meta {Metadata} The current metadata to use to find the parent node
 */
export declare const hasNestedTemplateLiteralsWithConditionalRules: (node: t.TemplateLiteral, meta: Metadata) => boolean;
/**
 * Manipulates template literal so that it resembles
 * CSS `property: value` declaration where
 * - before: all the CSS _before_ the interpolation
 * - after: all the CSS _after_ the interpolation
 * @param expression
 * @param quasi
 * @param nextQuasis
 */
export declare const moveCssPropertyInExpression: (expression: t.ConditionalExpression, quasi: t.TemplateElement, nextQuasis: t.TemplateElement, deleteValue?: boolean) => void;
