"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.moveCssPropertyInExpression = exports.hasNestedTemplateLiteralsWithConditionalRules = exports.isCssPropertyInTemplateElement = void 0;
var traverse_1 = __importDefault(require("@babel/traverse"));
var t = __importStar(require("@babel/types"));
var css_1 = require("@compiled/css");
var ast_1 = require("./ast");
var conditionalPaths = ['consequent', 'alternate'];
/**
 * Returns `true` if the CSS property or pseudo classes/pseudo elements
 * are defined within a template element (the string pieces of the template literal)
 * rather than in the expression.
 *
 * For example
 * - CSS property: `color: ${({ isPrimary }) => (isPrimary ? 'green' : 'red')};`
 * - Pseudo class: `hover: ${({ isPrimary }) => (isPrimary ? 'color:green' : 'color:red')};`
 * - Pseudo element: `:before {${({ isPrimary }) => (isPrimary ? 'color: green' : 'color:red')}};`
 * @param node
 */
var isCssPropertyInTemplateElement = function (node) {
    var value = node.value.raw;
    return value.includes(':');
};
exports.isCssPropertyInTemplateElement = isCssPropertyInTemplateElement;
/**
 * TODO: this is a temporary workaround so that we don't evaluate expressions that may throw an error.
 * It should be removed after addressing https://github.com/atlassian-labs/compiled/issues/1081
 *
 * Returns `true` if
 * - there is another conditional expression at the root of the current template literal
 *   with a conditional expression to evaluate OR
 * - there is a nested logical expression within the current template literal
 *
 * @param node
 * @param meta {Metadata} The current metadata to use to find the parent node
 */
var hasNestedTemplateLiteralsWithConditionalRules = function (node, meta) {
    var expressionPath = (0, ast_1.getPathOfNode)(node, meta.parentPath);
    var parent = expressionPath.parent;
    var isNested = false;
    (0, traverse_1.default)(parent, {
        noScope: true,
        ConditionalExpression: function (path) {
            conditionalPaths.map(function (c) {
                var expression = path.node[c];
                if ((t.isTaggedTemplateExpression(expression) && expression.quasi === node) ||
                    (t.isTemplateLiteral(expression) &&
                        expression.expressions.some(function (expressionNode) {
                            return t.isArrowFunctionExpression(expressionNode);
                        })) ||
                    t.isLogicalExpression(expression)) {
                    isNested = true;
                    path.stop();
                }
            });
        },
    });
    return isNested;
};
exports.hasNestedTemplateLiteralsWithConditionalRules = hasNestedTemplateLiteralsWithConditionalRules;
/**
 * Manipulates template literal so that it resembles
 * CSS `property: value` declaration where
 * - before: all the CSS _before_ the interpolation
 * - after: all the CSS _after_ the interpolation
 * @param expression
 * @param quasi
 * @param nextQuasis
 */
var moveCssPropertyInExpression = function (expression, quasi, nextQuasis, deleteValue) {
    var _a;
    if (deleteValue === void 0) { deleteValue = true; }
    var value = quasi.value.raw;
    var parts = value.split(';');
    var before = parts[parts.length - 1]; // everything after the last ';'. Hence, after the last CSS declaration
    var _b = (0, css_1.cssAffixInterpolation)('', nextQuasis.value.raw), nextQuasisExtractedCss = _b[1];
    var closingBracket = before.includes('{') ? ';}' : '';
    var after = nextQuasisExtractedCss.variableSuffix + closingBracket;
    if (!before) {
        return;
    }
    conditionalPaths.map(function (path) {
        var pathNode = expression[path];
        if (t.isStringLiteral(pathNode) && pathNode.value) {
            // We've found a string literal like: `'blue'`
            pathNode.value = before + pathNode.value + after;
        }
        else if (t.isTemplateLiteral(pathNode)) {
            // We've found a template literal like: "`${fontSize}px`"
            pathNode.quasis[0].value.raw = before + pathNode.quasis[0].value.raw + after;
        }
        else if (t.isNumericLiteral(pathNode)) {
            // We've found a numeric literal like: `1`
            expression[path] = t.stringLiteral(before + pathNode.value + after);
        }
        else if (t.isIdentifier(pathNode)) {
            var identifierExpression = [pathNode];
            var identifierQuasis = [
                t.templateElement({ raw: before, cooked: before }),
                t.templateElement({ raw: after, cooked: after }),
            ];
            expression[path] = t.templateLiteral(identifierQuasis, identifierExpression);
        }
        else if (t.isMemberExpression(pathNode)) {
            // We've found a member expression like `colors.N50`
            var memberExpression = [pathNode];
            var memberQuasis = [
                t.templateElement({ raw: before, cooked: before }),
                t.templateElement({ raw: after, cooked: after }),
            ];
            expression[path] = t.templateLiteral(memberQuasis, memberExpression);
        }
        else if (t.isConditionalExpression(pathNode)) {
            // We've found nested ternary operators
            (0, exports.moveCssPropertyInExpression)(pathNode, quasi, nextQuasis, false);
        }
        else {
            deleteValue = false;
            return;
        }
    });
    // Remove CSS property so it doesn't get processed again
    if (deleteValue) {
        var nextQuasisNewCSS = nextQuasisExtractedCss.css;
        quasi.value.raw = value.replace(before, '');
        quasi.value.cooked = value.replace(before, '');
        if (closingBracket) {
            var rex = /([^}/]+$)/g; // everything after the last '}'
            nextQuasisNewCSS = ((_a = nextQuasis.value.raw.match(rex)) === null || _a === void 0 ? void 0 : _a[0]) || '';
        }
        nextQuasis.value.raw = nextQuasisNewCSS;
        nextQuasis.value.cooked = nextQuasisNewCSS;
    }
};
exports.moveCssPropertyInExpression = moveCssPropertyInExpression;
//# sourceMappingURL=manipulate-template-literal.js.map